/*
 * Copyright (c) 2005 Zauber  -- All rights reserved
 */
package ar.com.zauber.taglibs.dateutlils;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;


/**
 * Utility class that formats a date in a relative format.
 * For example: 5 days ago
 *
 * @author Juan F. Codagnone
 * @since Nov 13, 2005
 */
public class RelativeDateFormatter {
    /** date used to do comparations */
    private final Date basedate;

    /** factors */
    private static DateFactor [] factor = {
            new DateFactor(60, TimeUnit.SECOND),
            new DateFactor(60, TimeUnit.MINUTE),
            new DateFactor(24, TimeUnit.HOUR),
            new DateFactor(7,  TimeUnit.DAY),
            new DateFactor(52, TimeUnit.WEEK),
    };
    
    /**
     * use now as baseDate
     *  
     * @see #RelativeDateFormatter(Date)
     */
    public RelativeDateFormatter() {
        this(new Date());
    }
    
    /**
     * Creates the RelativeDateFormatter.
     *
     * @param basedate date used as a base
     */
    public RelativeDateFormatter(final Date basedate) {
        this.basedate = basedate;
    }
    
    /**
     * @param locate the language
     * @param date date to format
     * @return a relative expression for date
     */
    public final String format(final String locate, final Date date) {
        final RelativeDateFormatterStringBuilder sb 
            = new RelativeDateFormatterStringBuilder(locate);
        
        final Calendar calendar = GregorianCalendar.getInstance();
        calendar.setTime(basedate);
        
        boolean past = basedate.after(date);
        float diff = (Math.max(date.getTime(), basedate.getTime())
                      - Math.min(date.getTime(), basedate.getTime())) / 1000;

        int i = 0;
        for( ; i < factor.length && diff > factor[i].getFactor() - 1; i++) {
            diff /= factor[i].getFactor(); 
        }

        
        if(past) {
            sb.setPast();
        } else {
            sb.setFuture();
        }
        
        long ret = Math.round(diff);
        sb.setAmount(ret);
        
        if(i >= factor.length) {
            sb.setTimeUnit(TimeUnit.YEAR);
        } else {
            sb.setTimeUnit(factor[i].getUnit());
        }
        
        return sb.build();
    }
}

/**
 * Time units
 * 
 * @author Pablo Martin Grigolatto
 * @since Jul 23, 2010
 */
enum TimeUnit {
    /** units */
    SECOND, MINUTE, HOUR, DAY, WEEK, YEAR;
}

/** helper */
class DateFactor {
    private final int factor;
    private final TimeUnit unit;
    
    /**
     * Creates the DateFactor.
     *
     * @param factor ...
     * @param unit the unit of time
     */
    public DateFactor(final int factor, final TimeUnit unit) {
        this.factor = factor;
        this.unit = unit;
    }

    public int getFactor() {
        return factor;
    }
    
    public TimeUnit getUnit() {
        return unit;
    }
}